@@ -51,9 +51,10 @@ use ibc_proto::cosmos::upgrade::v1beta1::{
     QueryCurrentPlanRequest, QueryUpgradedConsensusStateRequest,
 };
 use ibc_proto::ibc::core::channel::v1::{
-    PacketState, QueryChannelsRequest, QueryConnectionChannelsRequest,
-    QueryNextSequenceReceiveRequest, QueryPacketAcknowledgementsRequest,
-    QueryPacketCommitmentsRequest, QueryUnreceivedAcksRequest, QueryUnreceivedPacketsRequest,
+    PacketState, QueryChannelClientStateRequest, QueryChannelsRequest,
+    QueryConnectionChannelsRequest, QueryNextSequenceReceiveRequest,
+    QueryPacketAcknowledgementsRequest, QueryPacketCommitmentsRequest, QueryUnreceivedAcksRequest,
+    QueryUnreceivedPacketsRequest,
 };
 use ibc_proto::ibc::core::client::v1::{QueryClientStatesRequest, QueryConsensusStatesRequest};
 use ibc_proto::ibc::core::commitment::v1::MerkleProof;
@@ -113,7 +114,7 @@ impl CosmosSdkChain {
             .unbonding_time
             .ok_or_else(|| Kind::Grpc.context("none unbonding time".to_string()))?;
 
-        Ok(Duration::from_secs(res.seconds as u64))
+        Ok(Duration::new(res.seconds as u64, res.nanos as u32))
     }
 
     fn rpc_client(&self) -> &HttpClient {
@@ -146,7 +147,7 @@ impl CosmosSdkChain {
 
         let key = self
             .keybase()
-            .get_key()
+            .get_key(&self.config.key_name)
             .map_err(|e| Kind::KeyBase.context(e))?;
 
         // Create TxBody
@@ -216,7 +217,7 @@ impl CosmosSdkChain {
         // Sign doc and broadcast
         let signed = self
             .keybase
-            .sign_msg(signdoc_buf)
+            .sign_msg(&self.config.key_name, signdoc_buf)
             .map_err(|e| Kind::KeyBase.context(e))?;
 
         let tx_raw = TxRaw {
@@ -326,8 +327,8 @@ impl Chain for CosmosSdkChain {
             .map_err(|e| Kind::Rpc(config.rpc_addr.clone()).context(e))?;
 
         // Initialize key store and load key
-        let keybase =
-            KeyRing::new(Store::Test, config.clone()).map_err(|e| Kind::KeyBase.context(e))?;
+        let keybase = KeyRing::new(Store::Test, &config.account_prefix, &config.id)
+            .map_err(|e| Kind::KeyBase.context(e))?;
 
         let grpc_addr =
             Uri::from_str(&config.grpc_addr.to_string()).map_err(|e| Kind::Grpc.context(e))?;
@@ -430,7 +431,7 @@ impl Chain for CosmosSdkChain {
         // Get the key from key seed file
         let key = self
             .keybase()
-            .get_key()
+            .get_key(&self.config.key_name)
             .map_err(|e| Kind::KeyBase.context(e))?;
 
         let bech32 = encode_to_bech32(&key.address.to_hex(), &self.config.account_prefix)?;
@@ -444,7 +445,7 @@ impl Chain for CosmosSdkChain {
         // Get the key from key seed file
         let key = self
             .keybase()
-            .get_key()
+            .get_key(&self.config.key_name)
             .map_err(|e| Kind::KeyBase.context(e))?;
 
         Ok(key)
@@ -502,12 +503,20 @@ impl Chain for CosmosSdkChain {
             .map_err(|e| Kind::Grpc.context(e))?
             .into_inner();
 
-        let clients = response
+        // Deserialize into domain type
+        let mut clients: Vec<IdentifiedAnyClientState> = response
             .client_states
             .into_iter()
             .filter_map(|cs| IdentifiedAnyClientState::try_from(cs).ok())
             .collect();
 
+        // Sort by client identifier counter
+        clients.sort_by(|a, b| {
+            client_id_suffix(&a.client_id)
+                .unwrap_or(0) // Fallback to `0` suffix (no sorting) if client id is malformed
+                .cmp(&client_id_suffix(&b.client_id).unwrap_or(0))
+        });
+
         Ok(clients)
     }
 
@@ -742,14 +751,16 @@ impl Chain for CosmosSdkChain {
         height: ICSHeight,
     ) -> Result<ConnectionEnd, Error> {
         let res = self.query(Path::Connections(connection_id.clone()), height, false)?;
-        Ok(ConnectionEnd::decode_vec(&res.value)
-            .map_err(|e| Kind::Query(format!("connection {}", connection_id)).context(e))?)
+        let connection_end = ConnectionEnd::decode_vec(&res.value)
+            .map_err(|e| Kind::Query(format!("connection '{}'", connection_id)).context(e))?;
+
+        Ok(connection_end)
     }
 
     fn query_connection_channels(
         &self,
         request: QueryConnectionChannelsRequest,
-    ) -> Result<Vec<ChannelId>, Error> {
+    ) -> Result<Vec<IdentifiedChannelEnd>, Error> {
         crate::time!("query_connection_channels");
 
         let mut client = self
@@ -770,13 +781,12 @@ impl Chain for CosmosSdkChain {
         // TODO: add warnings for any identifiers that fail to parse (below).
         //  https://github.com/informalsystems/ibc-rs/pull/506#discussion_r555945560
 
-        let vec_ids = response
+        let channels = response
             .channels
-            .iter()
-            .filter_map(|ic| ChannelId::from_str(ic.channel_id.as_str()).ok())
+            .into_iter()
+            .filter_map(|ch| IdentifiedChannelEnd::try_from(ch).ok())
             .collect();
-
-        Ok(vec_ids)
+        Ok(channels)
     }
 
     fn query_channels(
@@ -819,8 +829,39 @@ impl Chain for CosmosSdkChain {
             height,
             false,
         )?;
-        Ok(ChannelEnd::decode_vec(&res.value)
-            .map_err(|e| Kind::Query("channel".into()).context(e))?)
+        let channel_end = ChannelEnd::decode_vec(&res.value).map_err(|e| {
+            Kind::Query(format!("port '{}' & channel '{}'", port_id, channel_id)).context(e)
+        })?;
+
+        Ok(channel_end)
+    }
+
+    fn query_channel_client_state(
+        &self,
+        request: QueryChannelClientStateRequest,
+    ) -> Result<Option<IdentifiedAnyClientState>, Error> {
+        crate::time!("query_channel_client_state");
+
+        let mut client = self
+            .block_on(
+                ibc_proto::ibc::core::channel::v1::query_client::QueryClient::connect(
+                    self.grpc_addr.clone(),
+                ),
+            )
+            .map_err(|e| Kind::Grpc.context(e))?;
+
+        let request = tonic::Request::new(request);
+
+        let response = self
+            .block_on(client.channel_client_state(request))
+            .map_err(|e| Kind::Grpc.context(e))?
+            .into_inner();
+
+        let client_state: Option<IdentifiedAnyClientState> = response
+            .identified_client_state
+            .map_or_else(|| None, |proto_cs| proto_cs.try_into().ok());
+
+        Ok(client_state)
     }
 
     /// Queries the packet commitment hashes associated with a channel.
@@ -1492,3 +1533,15 @@ fn encode_to_bech32(address: &str, account_prefix: &str) -> Result<String, Error
 
     Ok(encoded)
 }
+
+/// Returns the suffix counter for a CosmosSDK client id.
+/// Returns `None` if the client identifier is malformed
+/// and the suffix could not be parsed.
+pub fn client_id_suffix(client_id: &ClientId) -> Option<u64> {
+    client_id
+        .as_str()
+        .split('-')
+        .last()
+        .map(|e| e.parse::<u64>().ok())
+        .flatten()
+}
