@@ -1,37 +1,46 @@
-use std::fmt::Debug;
-use std::sync::Arc;
+use std::{
+    fmt::{self, Debug},
+    sync::Arc,
+};
 
 use crossbeam_channel as channel;
 use dyn_clone::DynClone;
 use serde::{Serialize, Serializer};
 
-use ibc::ics02_client::client_consensus::{AnyConsensusState, AnyConsensusStateWithHeight};
-use ibc::ics02_client::client_state::AnyClientState;
-use ibc::ics02_client::events::UpdateClient;
-use ibc::ics02_client::misbehaviour::AnyMisbehaviour;
-use ibc::ics04_channel::channel::IdentifiedChannelEnd;
-use ibc::query::QueryTxRequest;
 use ibc::{
     events::IbcEvent,
-    ics02_client::header::AnyHeader,
+    ics02_client::{
+        client_consensus::{AnyConsensusState, AnyConsensusStateWithHeight},
+        client_state::{AnyClientState, IdentifiedAnyClientState},
+        events::UpdateClient,
+        header::AnyHeader,
+        misbehaviour::AnyMisbehaviour,
+    },
     ics03_connection::{connection::ConnectionEnd, version::Version},
     ics04_channel::{
-        channel::ChannelEnd,
+        channel::{ChannelEnd, IdentifiedChannelEnd},
         packet::{PacketMsgType, Sequence},
     },
     ics23_commitment::commitment::CommitmentPrefix,
     ics24_host::identifier::{ChainId, ChannelId, ClientId, ConnectionId, PortId},
     proofs::Proofs,
+    query::QueryTxRequest,
     signer::Signer,
     Height,
 };
-use ibc_proto::ibc::core::channel::v1::{
-    PacketState, QueryChannelsRequest, QueryNextSequenceReceiveRequest,
-    QueryPacketAcknowledgementsRequest, QueryPacketCommitmentsRequest, QueryUnreceivedAcksRequest,
-    QueryUnreceivedPacketsRequest,
+
+use ibc_proto::ibc::core::{
+    channel::v1::{
+        PacketState, QueryChannelClientStateRequest, QueryChannelsRequest,
+        QueryConnectionChannelsRequest, QueryNextSequenceReceiveRequest,
+        QueryPacketAcknowledgementsRequest, QueryPacketCommitmentsRequest,
+        QueryUnreceivedAcksRequest, QueryUnreceivedPacketsRequest,
+    },
+    client::v1::{QueryClientStatesRequest, QueryConsensusStatesRequest},
+    commitment::v1::MerkleProof,
+    connection::v1::QueryClientConnectionsRequest,
 };
-use ibc_proto::ibc::core::client::v1::QueryConsensusStatesRequest;
-use ibc_proto::ibc::core::commitment::v1::MerkleProof;
+
 pub use prod::ProdChainHandle;
 
 use crate::{
@@ -43,6 +52,32 @@ use crate::{
 
 mod prod;
 
+/// A pair of [`ChainHandle`]s.
+#[derive(Clone)]
+pub struct ChainHandlePair {
+    pub a: Box<dyn ChainHandle>,
+    pub b: Box<dyn ChainHandle>,
+}
+
+impl ChainHandlePair {
+    /// Swap the two handles.
+    pub fn swap(self) -> Self {
+        Self {
+            a: self.b,
+            b: self.a,
+        }
+    }
+}
+
+impl Debug for ChainHandlePair {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("ChainHandlePair")
+            .field("a", &self.a.id())
+            .field("b", &self.b.id())
+            .finish()
+    }
+}
+
 pub type Subscription = channel::Receiver<Arc<MonitorResult<EventBatch>>>;
 
 pub type ReplyTo<T> = channel::Sender<Result<T, Error>>;
@@ -86,6 +121,11 @@ pub enum ChainRequest {
         reply_to: ReplyTo<Height>,
     },
 
+    QueryClients {
+        request: QueryClientStatesRequest,
+        reply_to: ReplyTo<Vec<IdentifiedAnyClientState>>,
+    },
+
     BuildHeader {
         trusted_height: Height,
         target_height: Height,
@@ -125,6 +165,11 @@ pub enum ChainRequest {
         reply_to: ReplyTo<AnyClientState>,
     },
 
+    QueryClientConnections {
+        request: QueryClientConnectionsRequest,
+        reply_to: ReplyTo<Vec<ConnectionId>>,
+    },
+
     QueryConsensusStates {
         request: QueryConsensusStatesRequest,
         reply_to: ReplyTo<Vec<AnyConsensusStateWithHeight>>,
@@ -161,6 +206,11 @@ pub enum ChainRequest {
         reply_to: ReplyTo<ConnectionEnd>,
     },
 
+    QueryConnectionChannels {
+        request: QueryConnectionChannelsRequest,
+        reply_to: ReplyTo<Vec<IdentifiedChannelEnd>>,
+    },
+
     QueryChannels {
         request: QueryChannelsRequest,
         reply_to: ReplyTo<Vec<IdentifiedChannelEnd>>,
@@ -173,6 +223,11 @@ pub enum ChainRequest {
         reply_to: ReplyTo<ChannelEnd>,
     },
 
+    QueryChannelClientState {
+        request: QueryChannelClientStateRequest,
+        reply_to: ReplyTo<Option<IdentifiedAnyClientState>>,
+    },
+
     QueryNextSequenceReceive {
         request: QueryNextSequenceReceiveRequest,
         reply_to: ReplyTo<Sequence>,
@@ -258,12 +313,22 @@ pub trait ChainHandle: DynClone + Send + Sync + Debug {
 
     fn query_latest_height(&self) -> Result<Height, Error>;
 
+    fn query_clients(
+        &self,
+        request: QueryClientStatesRequest,
+    ) -> Result<Vec<IdentifiedAnyClientState>, Error>;
+
     fn query_client_state(
         &self,
         client_id: &ClientId,
         height: Height,
     ) -> Result<AnyClientState, Error>;
 
+    fn query_client_connections(
+        &self,
+        request: QueryClientConnectionsRequest,
+    ) -> Result<Vec<ConnectionId>, Error>;
+
     fn query_consensus_states(
         &self,
         request: QueryConsensusStatesRequest,
@@ -296,6 +361,11 @@ pub trait ChainHandle: DynClone + Send + Sync + Debug {
         height: Height,
     ) -> Result<ConnectionEnd, Error>;
 
+    fn query_connection_channels(
+        &self,
+        request: QueryConnectionChannelsRequest,
+    ) -> Result<Vec<IdentifiedChannelEnd>, Error>;
+
     fn query_next_sequence_receive(
         &self,
         request: QueryNextSequenceReceiveRequest,
@@ -313,6 +383,11 @@ pub trait ChainHandle: DynClone + Send + Sync + Debug {
         height: Height,
     ) -> Result<ChannelEnd, Error>;
 
+    fn query_channel_client_state(
+        &self,
+        request: QueryChannelClientStateRequest,
+    ) -> Result<Option<IdentifiedAnyClientState>, Error>;
+
     fn proven_client_state(
         &self,
         client_id: &ClientId,
