@@ -4,19 +4,18 @@ use crossbeam_channel as channel;
 use tokio::runtime::Runtime as TokioRuntime;
 use tracing::error;
 
-use ibc::ics04_channel::channel::IdentifiedChannelEnd;
 use ibc::{
     events::IbcEvent,
     ics02_client::{
         client_consensus::{AnyConsensusState, AnyConsensusStateWithHeight, ConsensusState},
-        client_state::{AnyClientState, ClientState},
+        client_state::{AnyClientState, ClientState, IdentifiedAnyClientState},
         events::UpdateClient,
         header::{AnyHeader, Header},
         misbehaviour::AnyMisbehaviour,
     },
     ics03_connection::{connection::ConnectionEnd, version::Version},
     ics04_channel::{
-        channel::ChannelEnd,
+        channel::{ChannelEnd, IdentifiedChannelEnd},
         packet::{PacketMsgType, Sequence},
     },
     ics23_commitment::commitment::CommitmentPrefix,
@@ -26,15 +25,16 @@ use ibc::{
     signer::Signer,
     Height,
 };
-
 use ibc_proto::ibc::core::{
     channel::v1::{
-        PacketState, QueryChannelsRequest, QueryNextSequenceReceiveRequest,
+        PacketState, QueryChannelClientStateRequest, QueryChannelsRequest,
+        QueryConnectionChannelsRequest, QueryNextSequenceReceiveRequest,
         QueryPacketAcknowledgementsRequest, QueryPacketCommitmentsRequest,
         QueryUnreceivedAcksRequest, QueryUnreceivedPacketsRequest,
     },
-    client::v1::QueryConsensusStatesRequest,
+    client::v1::{QueryClientStatesRequest, QueryConsensusStatesRequest},
     commitment::v1::MerkleProof,
+    connection::v1::QueryClientConnectionsRequest,
 };
 
 use crate::{
@@ -168,15 +168,18 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
                         Ok(event_batch) => {
                             self.event_bus
                                 .broadcast(Arc::new(event_batch))
-                                .map_err(|e| Kind::Channel.context(e))?;
+                                .map_err(Kind::channel)?;
+                        },
+                        Err(e) => {
+                            error!("received error via event bus: {}", e);
+                            return Err(Kind::Channel.into());
                         },
-                        Err(e) => error!("received error via event bus: {}", e),
                     }
                 },
                 recv(self.request_receiver) -> event => {
                     match event {
                         Ok(ChainRequest::Terminate { reply_to }) => {
-                            reply_to.send(Ok(())).map_err(|_| Kind::Channel)?;
+                            reply_to.send(Ok(())).map_err(Kind::channel)?;
                             break;
                         }
 
@@ -228,6 +231,14 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
                             self.query_latest_height(reply_to)?
                         }
 
+                        Ok(ChainRequest::QueryClients { request, reply_to }) => {
+                            self.query_clients(request, reply_to)?
+                        },
+
+                        Ok(ChainRequest::QueryClientConnections { request, reply_to }) => {
+                            self.query_client_connections(request, reply_to)?
+                        },
+
                         Ok(ChainRequest::QueryClientState { client_id, height, reply_to }) => {
                             self.query_client_state(client_id, height, reply_to)?
                         },
@@ -260,6 +271,10 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
                             self.query_connection(connection_id, height, reply_to)?
                         },
 
+                        Ok(ChainRequest::QueryConnectionChannels { request, reply_to }) => {
+                            self.query_connection_channels(request, reply_to)?
+                        },
+
                         Ok(ChainRequest::QueryChannels { request, reply_to }) => {
                             self.query_channels(request, reply_to)?
                         },
@@ -268,6 +283,10 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
                             self.query_channel(port_id, channel_id, height, reply_to)?
                         },
 
+                        Ok(ChainRequest::QueryChannelClientState { request, reply_to }) => {
+                            self.query_channel_client_state(request, reply_to)?
+                        },
+
                         Ok(ChainRequest::ProvenClientState { client_id, height, reply_to }) => {
                             self.proven_client_state(client_id, height, reply_to)?
                         },
@@ -320,9 +339,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     fn subscribe(&mut self, reply_to: ReplyTo<Subscription>) -> Result<(), Error> {
         let subscription = self.event_bus.subscribe();
 
-        reply_to
-            .send(Ok(subscription))
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(Ok(subscription)).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -334,9 +351,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     ) -> Result<(), Error> {
         let result = self.chain.send_msgs(proto_msgs);
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -344,9 +359,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     fn query_latest_height(&self, reply_to: ReplyTo<Height>) -> Result<(), Error> {
         let latest_height = self.chain.query_latest_height();
 
-        reply_to
-            .send(latest_height)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(latest_height).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -354,9 +367,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     fn get_signer(&mut self, reply_to: ReplyTo<Signer>) -> Result<(), Error> {
         let result = self.chain.get_signer();
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -364,9 +375,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     fn get_key(&mut self, reply_to: ReplyTo<KeyEntry>) -> Result<(), Error> {
         let result = self.chain.get_key();
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -374,9 +383,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     fn module_version(&self, port_id: PortId, reply_to: ReplyTo<String>) -> Result<(), Error> {
         let result = self.chain.query_module_version(&port_id);
 
-        reply_to
-            .send(Ok(result))
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(Ok(result)).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -411,9 +418,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
                 .map_or_else(Err, |header| Ok(header.wrap_any())),
         };
 
-        reply_to
-            .send(header)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(header).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -429,9 +434,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
             .build_client_state(height)
             .map(|cs| cs.wrap_any());
 
-        reply_to
-            .send(client_state)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(client_state).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -451,9 +454,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
             .build_consensus_state(light_block)
             .map(|cs| cs.wrap_any());
 
-        reply_to
-            .send(consensus_state)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(consensus_state).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -469,9 +470,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
             .light_client
             .check_misbehaviour(update_event, &client_state);
 
-        reply_to
-            .send(misbehaviour)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(misbehaviour).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -494,9 +493,31 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
         let result = result
             .map(|(opt_client_state, proofs)| (opt_client_state.map(|cs| cs.wrap_any()), proofs));
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
+
+        Ok(())
+    }
+
+    fn query_clients(
+        &self,
+        request: QueryClientStatesRequest,
+        reply_to: ReplyTo<Vec<IdentifiedAnyClientState>>,
+    ) -> Result<(), Error> {
+        let result = self.chain.query_clients(request);
+
+        reply_to.send(result).map_err(Kind::channel)?;
+
+        Ok(())
+    }
+
+    fn query_client_connections(
+        &self,
+        request: QueryClientConnectionsRequest,
+        reply_to: ReplyTo<Vec<ConnectionId>>,
+    ) -> Result<(), Error> {
+        let result = self.chain.query_client_connections(request);
+
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -512,9 +533,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
             .query_client_state(&client_id, height)
             .map(|cs| cs.wrap_any());
 
-        reply_to
-            .send(client_state)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(client_state).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -529,9 +548,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
             .query_upgraded_client_state(height)
             .map(|(cl, proof)| (cl.wrap_any(), proof));
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -543,9 +560,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     ) -> Result<(), Error> {
         let consensus_states = self.chain.query_consensus_states(request);
 
-        reply_to
-            .send(consensus_states)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(consensus_states).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -561,9 +576,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
             self.chain
                 .query_consensus_state(client_id, consensus_height, query_height);
 
-        reply_to
-            .send(consensus_state)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(consensus_state).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -578,9 +591,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
             .query_upgraded_consensus_state(height)
             .map(|(cs, proof)| (cs.wrap_any(), proof));
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -588,9 +599,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     fn query_commitment_prefix(&self, reply_to: ReplyTo<CommitmentPrefix>) -> Result<(), Error> {
         let prefix = self.chain.query_commitment_prefix();
 
-        reply_to
-            .send(prefix)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(prefix).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -598,9 +607,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     fn query_compatible_versions(&self, reply_to: ReplyTo<Vec<Version>>) -> Result<(), Error> {
         let versions = self.chain.query_compatible_versions();
 
-        reply_to
-            .send(versions)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(versions).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -613,8 +620,20 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     ) -> Result<(), Error> {
         let connection_end = self.chain.query_connection(&connection_id, height);
 
+        reply_to.send(connection_end).map_err(Kind::channel)?;
+
+        Ok(())
+    }
+
+    fn query_connection_channels(
+        &self,
+        request: QueryConnectionChannelsRequest,
+        reply_to: ReplyTo<Vec<IdentifiedChannelEnd>>,
+    ) -> Result<(), Error> {
+        let result = self.chain.query_connection_channels(request);
+
         reply_to
-            .send(connection_end)
+            .send(result)
             .map_err(|e| Kind::Channel.context(e))?;
 
         Ok(())
@@ -627,9 +646,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     ) -> Result<(), Error> {
         let result = self.chain.query_channels(request);
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -643,9 +660,19 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     ) -> Result<(), Error> {
         let result = self.chain.query_channel(&port_id, &channel_id, height);
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
+
+        Ok(())
+    }
+
+    fn query_channel_client_state(
+        &self,
+        request: QueryChannelClientStateRequest,
+        reply_to: ReplyTo<Option<IdentifiedAnyClientState>>,
+    ) -> Result<(), Error> {
+        let result = self.chain.query_channel_client_state(request);
+
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -661,9 +688,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
             .proven_client_state(&client_id, height)
             .map(|(cs, mp)| (cs.wrap_any(), mp));
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -676,9 +701,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     ) -> Result<(), Error> {
         let result = self.chain.proven_connection(&connection_id, height);
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -695,9 +718,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
             .proven_client_consensus(&client_id, consensus_height, height)
             .map(|(cs, mp)| (cs.wrap_any(), mp));
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -713,9 +734,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
             .chain
             .build_channel_proofs(&port_id, &channel_id, height);
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -733,9 +752,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
             self.chain
                 .build_packet_proofs(packet_type, port_id, channel_id, sequence, height);
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -747,9 +764,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     ) -> Result<(), Error> {
         let result = self.chain.query_packet_commitments(request);
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -761,9 +776,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     ) -> Result<(), Error> {
         let result = self.chain.query_unreceived_packets(request);
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -775,9 +788,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     ) -> Result<(), Error> {
         let result = self.chain.query_packet_acknowledgements(request);
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -789,9 +800,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     ) -> Result<(), Error> {
         let result = self.chain.query_unreceived_acknowledgements(request);
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -803,9 +812,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     ) -> Result<(), Error> {
         let result = self.chain.query_next_sequence_receive(request);
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
@@ -817,9 +824,7 @@ impl<C: Chain + Send + 'static> ChainRuntime<C> {
     ) -> Result<(), Error> {
         let result = self.chain.query_txs(request);
 
-        reply_to
-            .send(result)
-            .map_err(|e| Kind::Channel.context(e))?;
+        reply_to.send(result).map_err(Kind::channel)?;
 
         Ok(())
     }
