@@ -15,17 +15,19 @@ use ibc::ics03_connection::msgs::conn_open_confirm::MsgConnectionOpenConfirm;
 use ibc::ics03_connection::msgs::conn_open_init::MsgConnectionOpenInit;
 use ibc::ics03_connection::msgs::conn_open_try::MsgConnectionOpenTry;
 use ibc::ics24_host::identifier::{ChainId, ClientId, ConnectionId};
+use ibc::timestamp::ZERO_DURATION;
 use ibc::tx_msg::Msg;
 use ibc::Height as ICSHeight;
 
 use crate::chain::handle::ChainHandle;
 use crate::error::Error;
 use crate::foreign_client::{ForeignClient, ForeignClientError};
-use crate::relay::MAX_ITER;
 
 /// Maximum value allowed for packet delay on any new connection that the relayer establishes.
 pub const MAX_PACKET_DELAY: Duration = Duration::from_secs(120);
 
+const MAX_RETRIES: usize = 5;
+
 #[derive(Debug, Error)]
 pub enum ConnectionError {
     #[error("failed with underlying cause: {0}")]
@@ -251,7 +253,7 @@ impl Connection {
 
         // Try connOpenInit on a_chain
         let mut counter = 0;
-        while counter < MAX_ITER {
+        while counter < MAX_RETRIES {
             counter += 1;
             match self.flipped().build_conn_init_and_send() {
                 Err(e) => {
@@ -268,7 +270,7 @@ impl Connection {
 
         // Try connOpenTry on b_chain
         counter = 0;
-        while counter < MAX_ITER {
+        while counter < MAX_RETRIES {
             counter += 1;
             match self.build_conn_try_and_send() {
                 Err(e) => {
@@ -284,7 +286,7 @@ impl Connection {
         }
 
         counter = 0;
-        while counter < MAX_ITER {
+        while counter < MAX_RETRIES {
             counter += 1;
 
             // Continue loop if query error
@@ -341,7 +343,7 @@ impl Connection {
 
         Err(ConnectionError::Failed(format!(
             "Failed to finish connection handshake in {:?} iterations",
-            MAX_ITER
+            MAX_RETRIES
         )))
     }
 
@@ -381,7 +383,7 @@ impl Connection {
             self.dst_client_id().clone(),
             counterparty,
             versions,
-            Duration::from_secs(0),
+            ZERO_DURATION,
         );
 
         // Retrieve existing connection if any
@@ -410,18 +412,14 @@ impl Connection {
     }
 
     pub fn build_update_client_on_src(&self, height: Height) -> Result<Vec<Any>, ConnectionError> {
-        let client =
-            ForeignClient::restore(self.src_client_id(), self.src_chain(), self.dst_chain());
-
+        let client = self.restore_src_client();
         client.build_update_client(height).map_err(|e| {
             ConnectionError::ClientOperation(self.src_client_id().clone(), self.src_chain().id(), e)
         })
     }
 
     pub fn build_update_client_on_dst(&self, height: Height) -> Result<Vec<Any>, ConnectionError> {
-        let client =
-            ForeignClient::restore(self.dst_client_id(), self.dst_chain(), self.src_chain());
-
+        let client = self.restore_dst_client();
         client.build_update_client(height).map_err(|e| {
             ConnectionError::ClientOperation(self.dst_client_id().clone(), self.dst_chain().id(), e)
         })
@@ -504,11 +502,11 @@ impl Connection {
         // Cross-check the delay_period
         let delay = if src_connection.delay_period() != self.delay_period {
             warn!("`delay_period` for ConnectionEnd @{} is {}s; delay period on local Connection object is set to {}s",
-                self.src_chain().id(), src_connection.delay_period().as_secs(), self.delay_period.as_secs());
+                self.src_chain().id(), src_connection.delay_period().as_secs_f64(), self.delay_period.as_secs_f64());
 
             warn!(
                 "Overriding delay period for local connection object to {}s",
-                src_connection.delay_period().as_secs()
+                src_connection.delay_period().as_secs_f64()
             );
 
             src_connection.delay_period()
@@ -799,6 +797,22 @@ impl Connection {
             _ => panic!("internal error"),
         }
     }
+
+    fn restore_src_client(&self) -> ForeignClient {
+        ForeignClient::restore(
+            self.src_client_id().clone(),
+            self.src_chain().clone(),
+            self.dst_chain().clone(),
+        )
+    }
+
+    fn restore_dst_client(&self) -> ForeignClient {
+        ForeignClient::restore(
+            self.dst_client_id().clone(),
+            self.dst_chain().clone(),
+            self.src_chain().clone(),
+        )
+    }
 }
 
 fn extract_connection_id(event: &IbcEvent) -> Result<&ConnectionId, ConnectionError> {
