@@ -26,10 +26,11 @@ use ibc::Height;
 use ibc_proto::ibc::core::client::v1::QueryConsensusStatesRequest;
 
 use crate::chain::handle::ChainHandle;
-use crate::relay::MAX_ITER;
 
 const MAX_MISBEHAVIOUR_CHECK_DURATION: Duration = Duration::from_secs(120);
 
+const MAX_RETRIES: usize = 5;
+
 #[derive(Debug, Error)]
 pub enum ForeignClientError {
     #[error("error raised while creating client: {0}")]
@@ -76,6 +77,13 @@ pub struct ForeignClient {
     pub src_chain: Box<dyn ChainHandle>,
 }
 
+/// Used in Output messages.
+/// Provides a concise description of a [`ForeignClient`],
+/// using the format:
+///     {CHAIN-ID} -> {CHAIN-ID}:{CLIENT}
+/// where the first chain identifier is for the source
+/// chain, and the second chain identifier is the
+/// destination (which hosts the client) chain.
 impl fmt::Display for ForeignClient {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(
@@ -117,14 +125,14 @@ impl ForeignClient {
     }
 
     pub fn restore(
-        client_id: &ClientId,
+        id: ClientId,
         dst_chain: Box<dyn ChainHandle>,
         src_chain: Box<dyn ChainHandle>,
     ) -> ForeignClient {
         ForeignClient {
-            id: client_id.clone(),
-            dst_chain: dst_chain.clone(),
-            src_chain: src_chain.clone(),
+            id,
+            dst_chain,
+            src_chain,
         }
     }
 
@@ -151,7 +159,7 @@ impl ForeignClient {
                 } else {
                     // TODO: Any additional checks?
                     Ok(ForeignClient::restore(
-                        client_id,
+                        client_id.clone(),
                         host_chain.clone(),
                         expected_target_chain,
                     ))
@@ -564,7 +572,7 @@ impl ForeignClient {
         };
 
         let mut events = vec![];
-        for i in 0..MAX_ITER {
+        for i in 0..MAX_RETRIES {
             thread::sleep(Duration::from_millis(100));
             let result = self
                 .dst_chain
@@ -583,7 +591,7 @@ impl ForeignClient {
                         self,
                         e,
                         i + 1,
-                        MAX_ITER
+                        MAX_RETRIES
                     );
                     continue;
                 }
@@ -912,9 +920,10 @@ pub fn extract_client_id(event: &IbcEvent) -> Result<&ClientId, ForeignClientErr
     match event {
         IbcEvent::CreateClient(ev) => Ok(ev.client_id()),
         IbcEvent::UpdateClient(ev) => Ok(ev.client_id()),
-        _ => Err(ForeignClientError::ClientCreate(
-            "cannot extract client_id from result".to_string(),
-        )),
+        other => Err(ForeignClientError::ClientCreate(format!(
+            "cannot extract client_id from result: {:?}",
+            other
+        ))),
     }
 }
 
@@ -926,6 +935,7 @@ mod test {
     use std::str::FromStr;
     use std::sync::Arc;
 
+    use test_env_log::test;
     use tokio::runtime::Runtime as TokioRuntime;
 
     use ibc::events::IbcEvent;
@@ -947,10 +957,10 @@ mod test {
         let (a_chain, _) = ChainRuntime::<MockChain>::spawn(a_cfg, rt.clone()).unwrap();
         let (b_chain, _) = ChainRuntime::<MockChain>::spawn(b_cfg, rt).unwrap();
         let a_client =
-            ForeignClient::restore(&Default::default(), a_chain.clone(), b_chain.clone());
+            ForeignClient::restore(ClientId::default(), a_chain.clone(), b_chain.clone());
 
         let b_client =
-            ForeignClient::restore(&Default::default(), b_chain.clone(), a_chain.clone());
+            ForeignClient::restore(ClientId::default(), b_chain.clone(), a_chain.clone());
 
         // Create the client on chain a
         let res = a_client.build_create_client_and_send();
@@ -984,10 +994,10 @@ mod test {
         let rt = Arc::new(TokioRuntime::new().unwrap());
         let (a_chain, _) = ChainRuntime::<MockChain>::spawn(a_cfg, rt.clone()).unwrap();
         let (b_chain, _) = ChainRuntime::<MockChain>::spawn(b_cfg, rt).unwrap();
-        let mut a_client = ForeignClient::restore(&a_client_id, a_chain.clone(), b_chain.clone());
+        let mut a_client = ForeignClient::restore(a_client_id, a_chain.clone(), b_chain.clone());
 
         let mut b_client =
-            ForeignClient::restore(&Default::default(), b_chain.clone(), a_chain.clone());
+            ForeignClient::restore(ClientId::default(), b_chain.clone(), a_chain.clone());
 
         // This action should fail because no client exists (yet)
         let res = a_client.build_latest_update_client_and_send();
