@@ -1,9 +1,13 @@
-pub mod errors;
-
-use std::convert::{TryFrom, TryInto};
-use std::fs::{self, File};
 use std::path::{Path, PathBuf};
 use std::str::FromStr;
+use std::{
+    collections::HashMap,
+    convert::{TryFrom, TryInto},
+};
+use std::{
+    ffi::OsStr,
+    fs::{self, File},
+};
 
 use bech32::{ToBase32, Variant};
 use bip39::{Language, Mnemonic, Seed};
@@ -13,17 +17,20 @@ use bitcoin::{
     util::bip32::{DerivationPath, ExtendedPrivKey, ExtendedPubKey},
 };
 use hdpath::StandardHDPath;
+use ibc::ics24_host::identifier::ChainId;
 use k256::ecdsa::{signature::Signer, Signature, SigningKey};
 use ripemd160::Ripemd160;
 use serde::{Deserialize, Serialize};
 use sha2::{Digest, Sha256};
 
-use crate::config::ChainConfig;
-
 use errors::{Error, Kind};
+pub use pub_key::EncodedPubKey;
+
+pub mod errors;
+mod pub_key;
 
 pub const KEYSTORE_DEFAULT_FOLDER: &str = ".hermes/keys/";
-pub const KEYSTORE_DISK_BACKEND: &str = "keyring-test"; // TODO: Change to "keyring"
+pub const KEYSTORE_DISK_BACKEND: &str = "keyring-test";
 pub const KEYSTORE_FILE_EXTENSION: &str = "json";
 
 /// [Coin type][coin-type] associated with a key.
@@ -98,37 +105,36 @@ impl TryFrom<KeyFile> for KeyEntry {
         // Decode the Bech32-encoded address from the key file
         let keyfile_address_bytes = decode_bech32(&key_file.address)?;
 
-        // Decode the Bech32-encoded public key from the key file
-        let mut keyfile_pubkey_bytes = decode_bech32(&key_file.pubkey)?;
+        let encoded_key: EncodedPubKey = key_file.pubkey.parse()?;
+        let mut keyfile_pubkey_bytes = encoded_key.into_bytes();
 
         // Use coin type if present or default coin type (ie. Atom).
         let coin_type = key_file.coin_type.unwrap_or_default();
 
         // Decode the private key from the mnemonic
         let private_key = private_key_from_mnemonic(&key_file.mnemonic, coin_type)?;
-        let public_key = ExtendedPubKey::from_private(&Secp256k1::new(), &private_key);
-        let public_key_bytes = public_key.public_key.to_bytes();
-
-        assert!(public_key_bytes.len() <= keyfile_pubkey_bytes.len());
+        let derived_pubkey = ExtendedPubKey::from_private(&Secp256k1::new(), &private_key);
+        let derived_pubkey_bytes = derived_pubkey.public_key.to_bytes();
+        assert!(derived_pubkey_bytes.len() <= keyfile_pubkey_bytes.len());
 
         // FIXME: For some reason that is currently unclear, the public key decoded from
         //        the keyfile contains a few extraneous leading bytes. To compare both
         //        public keys, we therefore strip those leading bytes off and keep the
         //        common parts.
         let keyfile_pubkey_bytes =
-            keyfile_pubkey_bytes.split_off(keyfile_pubkey_bytes.len() - public_key_bytes.len());
+            keyfile_pubkey_bytes.split_off(keyfile_pubkey_bytes.len() - derived_pubkey_bytes.len());
 
         // Ensure that the public key in the key file and the one extracted from the mnemonic match.
-        if keyfile_pubkey_bytes != public_key_bytes {
+        if keyfile_pubkey_bytes != derived_pubkey_bytes {
             return Err(Kind::PublicKeyMismatch {
                 keyfile: keyfile_pubkey_bytes,
-                mnemonic: public_key_bytes,
+                mnemonic: derived_pubkey_bytes,
             }
             .into());
         }
 
         Ok(KeyEntry {
-            public_key,
+            public_key: derived_pubkey,
             private_key,
             account: key_file.address,
             address: keyfile_address_bytes,
@@ -138,54 +144,62 @@ impl TryFrom<KeyFile> for KeyEntry {
 }
 
 pub trait KeyStore {
-    fn get_key(&self) -> Result<KeyEntry, Error>;
-    fn add_key(&mut self, key_entry: KeyEntry) -> Result<(), Error>;
+    fn get_key(&self, key_name: &str) -> Result<KeyEntry, Error>;
+    fn add_key(&mut self, key_name: &str, key_entry: KeyEntry) -> Result<(), Error>;
+    fn keys(&self) -> Result<Vec<(String, KeyEntry)>, Error>;
 }
 
 #[derive(Clone, Debug, Serialize, Deserialize)]
 pub struct Memory {
     account_prefix: String,
-    key_entry: Option<KeyEntry>,
+    keys: HashMap<String, KeyEntry>,
 }
 
 impl Memory {
-    pub fn new(account_prefix: String, key_entry: Option<KeyEntry>) -> Self {
+    pub fn new(account_prefix: String) -> Self {
         Self {
             account_prefix,
-            key_entry,
+            keys: HashMap::new(),
         }
     }
 }
 
 impl KeyStore for Memory {
-    fn get_key(&self) -> Result<KeyEntry, Error> {
-        self.key_entry
-            .clone()
+    fn get_key(&self, key_name: &str) -> Result<KeyEntry, Error> {
+        self.keys
+            .get(key_name)
+            .cloned()
             .ok_or_else(|| Kind::KeyNotFound.into())
     }
 
-    fn add_key(&mut self, key_entry: KeyEntry) -> Result<(), Error> {
-        if self.key_entry.is_some() {
+    fn add_key(&mut self, key_name: &str, key_entry: KeyEntry) -> Result<(), Error> {
+        if self.keys.contains_key(key_name) {
             return Err(Kind::ExistingKey.into());
         }
 
-        self.key_entry = Some(key_entry);
+        self.keys.insert(key_name.to_string(), key_entry);
 
         Ok(())
     }
+
+    fn keys(&self) -> Result<Vec<(String, KeyEntry)>, Error> {
+        Ok(self
+            .keys
+            .iter()
+            .map(|(n, k)| (n.to_string(), k.clone()))
+            .collect())
+    }
 }
 
 #[derive(Clone, Debug, Serialize, Deserialize)]
 pub struct Test {
-    key_name: String,
     account_prefix: String,
     store: PathBuf,
 }
 
 impl Test {
-    pub fn new(key_name: String, account_prefix: String, store: PathBuf) -> Self {
+    pub fn new(account_prefix: String, store: PathBuf) -> Self {
         Self {
-            key_name,
             account_prefix,
             store,
         }
@@ -193,25 +207,28 @@ impl Test {
 }
 
 impl KeyStore for Test {
-    fn get_key(&self) -> Result<KeyEntry, Error> {
-        let mut filename = self.store.join(&self.key_name);
-        filename.set_extension(KEYSTORE_FILE_EXTENSION);
+    fn get_key(&self, key_name: &str) -> Result<KeyEntry, Error> {
+        let mut key_file = self.store.join(key_name);
+        key_file.set_extension(KEYSTORE_FILE_EXTENSION);
 
-        if !filename.as_path().exists() {
-            return Err(Kind::KeyStore.context("cannot find key file").into());
+        if !key_file.as_path().exists() {
+            return Err(Kind::KeyStore
+                .context(format!("cannot find key file at '{}'", key_file.display()))
+                .into());
         }
 
         let file =
-            File::open(filename).map_err(|_| Kind::KeyStore.context("cannot open key file"))?;
+            File::open(&key_file).map_err(|_| Kind::KeyStore.context("cannot open key file"))?;
 
-        let key_entry = serde_json::from_reader(file)
-            .map_err(|_| Kind::KeyStore.context("cannot ready key file"))?;
+        let key_entry = serde_json::from_reader(file).map_err(|_| {
+            Kind::KeyStore.context(format!("cannot read key file at '{}'", key_file.display()))
+        })?;
 
         Ok(key_entry)
     }
 
-    fn add_key(&mut self, key_entry: KeyEntry) -> Result<(), Error> {
-        let mut filename = self.store.join(&self.key_name);
+    fn add_key(&mut self, key_name: &str, key_entry: KeyEntry) -> Result<(), Error> {
+        let mut filename = self.store.join(key_name);
         filename.set_extension(KEYSTORE_FILE_EXTENSION);
 
         let file = File::create(filename)
@@ -222,6 +239,22 @@ impl KeyStore for Test {
 
         Ok(())
     }
+
+    fn keys(&self) -> Result<Vec<(String, KeyEntry)>, Error> {
+        let dir = fs::read_dir(&self.store)
+            .map_err(|e| Kind::KeyStore.context(format!("cannot list keys: {}", e)))?;
+
+        let ext = OsStr::new(KEYSTORE_FILE_EXTENSION);
+
+        dir.into_iter()
+            .flatten()
+            .map(|entry| entry.path())
+            .filter(|path| path.extension() == Some(ext))
+            .flat_map(|path| path.file_stem().map(OsStr::to_owned))
+            .flat_map(|stem| stem.to_str().map(ToString::to_string))
+            .map(|name| self.get_key(&name).map(|key| (name, key)))
+            .collect()
+    }
 }
 
 #[derive(Copy, Clone, Debug, PartialEq, Eq, Hash)]
@@ -237,12 +270,12 @@ pub enum KeyRing {
 }
 
 impl KeyRing {
-    pub fn new(store: Store, chain_config: ChainConfig) -> Result<Self, Error> {
+    pub fn new(store: Store, account_prefix: &str, chain_id: &ChainId) -> Result<Self, Error> {
         match store {
-            Store::Memory => Ok(Self::Memory(Memory::new(chain_config.account_prefix, None))),
+            Store::Memory => Ok(Self::Memory(Memory::new(account_prefix.to_string()))),
 
             Store::Test => {
-                let keys_folder = disk_store_path(chain_config.id.as_str()).map_err(|e| {
+                let keys_folder = disk_store_path(chain_id.as_str()).map_err(|e| {
                     Kind::KeyStore.context(format!("failed to compute keys folder path: {:?}", e))
                 })?;
 
@@ -252,25 +285,31 @@ impl KeyRing {
                 })?;
 
                 Ok(Self::Test(Test::new(
-                    chain_config.key_name,
-                    chain_config.account_prefix,
+                    account_prefix.to_string(),
                     keys_folder,
                 )))
             }
         }
     }
 
-    pub fn get_key(&self) -> Result<KeyEntry, Error> {
+    pub fn get_key(&self, key_name: &str) -> Result<KeyEntry, Error> {
         match self {
-            KeyRing::Memory(m) => m.get_key(),
-            KeyRing::Test(d) => d.get_key(),
+            KeyRing::Memory(m) => m.get_key(key_name),
+            KeyRing::Test(d) => d.get_key(key_name),
         }
     }
 
-    pub fn add_key(&mut self, key_entry: KeyEntry) -> Result<(), Error> {
+    pub fn add_key(&mut self, key_name: &str, key_entry: KeyEntry) -> Result<(), Error> {
         match self {
-            KeyRing::Memory(m) => m.add_key(key_entry),
-            KeyRing::Test(d) => d.add_key(key_entry),
+            KeyRing::Memory(m) => m.add_key(key_name, key_entry),
+            KeyRing::Test(d) => d.add_key(key_name, key_entry),
+        }
+    }
+
+    pub fn keys(&self) -> Result<Vec<(String, KeyEntry)>, Error> {
+        match self {
+            KeyRing::Memory(m) => m.keys(),
+            KeyRing::Test(d) => d.keys(),
         }
     }
 
@@ -311,8 +350,8 @@ impl KeyRing {
     }
 
     /// Sign a message
-    pub fn sign_msg(&self, msg: Vec<u8>) -> Result<Vec<u8>, Error> {
-        let key = self.get_key()?;
+    pub fn sign_msg(&self, key_name: &str, msg: Vec<u8>) -> Result<Vec<u8>, Error> {
+        let key = self.get_key(key_name)?;
 
         let private_key_bytes = key.private_key.private_key.to_bytes();
         let signing_key = SigningKey::from_bytes(private_key_bytes.as_slice()).map_err(|_| {
