@@ -6,6 +6,9 @@
     unused_qualifications,
     rust_2018_idioms
 )]
+// TODO: disable unwraps:
+//  https://github.com/informalsystems/ibc-rs/issues/987
+// #![cfg_attr(not(test), deny(clippy::unwrap_used))]
 
 //! IBC Relayer implementation as a library.
 //!
@@ -24,9 +27,11 @@ pub mod keyring;
 pub mod light_client;
 pub mod link;
 pub mod macros;
+pub mod object;
 pub mod registry;
-pub mod relay;
 pub mod supervisor;
+pub mod telemetry;
 pub mod transfer;
 pub mod upgrade_chain;
 pub mod util;
+pub mod worker;
